struct model modelOptim(int model,double h[],double g[],int kSup)
{ 
	// Estimate the best parameters for the model
	//(inspired by Standard PSO 2007, http://www.particleswarm.info)

	struct model bestBest={0}; // Best position over all runs, and total error
	// Set to zero just for my stupid compiler
	struct model best;
	double error;			// Current error
	double errorMean;		// Average error
	int k;
	double hMin;
	int nFailure;		// Number of unsuccessful runs
	struct param param;
	struct problem pb; 
	int run, runMax;  
	//printf("\nmodelOptim 19, model %i",model);

	runMax = 10;
	// -----------------------------------------------------
	// PARAMETERS 
	param.initLink = 0; // 0 => re-init links after each unsuccessful iteration
	// 1 => re-init links after each successful iteration

	param.randOrder=1; // 1 = random permutation of particles before each iteratio 
	param.stop = 1;	// Stop criterion
	// 0 => error < pb.epsilon
	// 1 => eval < pb.evalMax		

	// Initialize some objects
	hMin=h[0]; hMax=h[0];
	gMax=0;
	for(k=1;k<=kSup;k++)
	{
		if(h[k]<hMin) hMin=h[k];
		if(h[k]>hMax) hMax=h[k];
		if(g[k]>gMax) gMax=g[k];
	}

	switch(model)
	{
		default:
			case 0: // c.c[0]*pow(h,c.c[1]);
				pb.evalMax=2000;
			pb.SS.D=2;
			pb.SS.min[0]=0; pb.SS.max[0]=gMax;
			pb.SS.min[1]=0; pb.SS.max[1]=6;
			break;

			case 1: // g=c.c[0]*(1-exp(-h/c.c[1]))
				pb.evalMax=2000;
			pb.SS.D=2;
			pb.SS.min[0]=0;  pb.SS.max[0]=gMax;
			pb.SS.min[1]=0; pb.SS.max[1]=10*hMax;
			break;

			case 2: // g=c.c[0]*(1-pow(1-h/c.c[1],c.c[2]))
				// g=c.c[0]*(1-pow((h-c.c[1])/(h-hMax),c.c[3]))
				pb.evalMax=4000;
			pb.SS.D=4;
			pb.SS.min[0]=0; pb.SS.max[0]=gMax;
			pb.SS.min[1]=0; pb.SS.max[1]=hMax;
			pb.SS.min[2]=0; pb.SS.max[2]=20;
			pb.SS.min[3]=0; pb.SS.max[3]=30;
			break;

			case 3: // Beta distribution
				pb.evalMax=2000;
			pb.SS.D=2;
			pb.SS.min[0]=0; pb.SS.max[0]=30; // alpha
			pb.SS.min[1]=0; pb.SS.max[1]=50; // beta
			break;
	}


	//  printf("\n\n Optimise model %i",model);
	/*
	 printf("\n evalMax %i",pb.evalMax);
	 printf("\n hMin %f, hMax %f",hMin,hMax);
	 printf("\n gMax %f",gMax);
	 printf("\nD %i",pb.SS.D);
	 printf("\n Search space");
	 for(d=0;d<pb.SS.D;d++) 
	 {
		 printf("\n%f %f",pb.SS.min[d],pb.SS.max[d]);
	 }
	 */

	pb.epsilon=0;
	pb.objective=0;
	pb.function=model;

	param.S = (int) (10 + 2 * sqrt(pb.SS.D));	// Swarm size
	param.K=3; 											
	param.p=1-pow(1-(double)1/(param.S),param.K); 


	// According to Clerc's Stagnation Analysis
	param.w[0] = 1 / (2 * log ((double) 2)); // 0.721
	param.c1 = 0.5 + log ((double) 2); // 1.193

	errorMean = 0;	        
	nFailure = 0;	
	//------------------------------------- RUNS	
	for (run = 0; run < runMax; run++)  
	{	
		//printf("\n");printf("run %i",run);

		best = modelOptimPSO (model,param, pb,h,g,kSup);
		error = best.error;

		// Memorize the best (useful if more than one run)
		if(run==0 || error<bestBest.error ) bestBest=best;
	}		// End loop on "run"


	//printf("\nmodel %i, Coefficients: ",bestBest.model);
	//for(d=0;d<pb.SS.D;d++) printf("%f ",bestBest.c[d]);

	/*
	 // Warnings about parameters
	 switch(model)
	 {
		 case 0: 
			 if(bestBest.c[1]>=pb.SS.max[1]-zero)
				 printf("\nWarning. Model 0: coeff 1 on the max limit %f",pb.SS.max[1]);
			 break;

		 case 1: 
			 if(bestBest.c[1]>=pb.SS.max[1]-zero)
				 printf("\nWarning. Model 1: coeff 1 on the max limit %f",pb.SS.max[1]);
			 break;

		 case 2:
			 for(d=2;d<=3;d++)
				 if(bestBest.c[d]>=pb.SS.max[d]-zero)
				 printf("\nWarning. Model 2: coeff %i on the max limit %f",d,pb.SS.max[d]);
			 break;

		 case 3: 
			 for(d=0;d<=1;d++)
				 if(bestBest.c[d]>=pb.SS.max[d]-zero)
				 printf("\nWarning. Model 3: coeff %i on the max limit %f",d,pb.SS.max[d]);
			 break;

	 }
	 */
	//printf("\n error %e",bestBest.error);

	// Save the coefficients
	// for(d=0;d<pb.SS.D;d++) fprintf(f_coeff,"%f ",bestBest.c[d]);
	//fprintf(f_coeff,"\n");

	return bestBest;
}
// ===============================================================
struct model modelOptimPSO (int model,struct param param, struct problem pb,double h[],double g[],int kSup) 
{ 
	// Estimate the best parameters thanks to a simple Particle Swarm Optimisation
	struct velocity aleaV;  
	int d;   
	double error=0;   // Set to zero just for my stupid compiler 
	double errorPrev;
	int gg; 
	struct velocity GX;   
	int index[SMax], indexTemp[SMax];     
	int initLinks;	// Flag to (re)init or not the information links
	int iter; 		// Iteration number (time step)
	int iterBegin;
	int length;
	int LINKS[SMax][SMax];	// Information links
	int m;
	int nEval;
	int noEval; 	
	int noStop;
	double p;
	struct velocity PX;	
	struct model R={0};
	int rank;
	struct swarmModel SW;
	int s0, s,s1; 
	int t;
	double zz;

	aleaV.size=pb.SS.D;
	// -----------------------------------------------------
	// INITIALISATION
	p=param.p; // Probability threshold for random topology
	SW.S=param.S;
	R.model=model;

	// Position and velocity
	for (s = 0; s < param.S; s++)   
	{ 
		SW.X[s].size = pb.SS.D;
		SW.X[s].model=model;
		SW.V[s].size = pb.SS.D;

		for (d = 0; d < pb.SS.D; d++)  
		{  
			SW.X[s].c[d] = alea (pb.SS.min[d], pb.SS.max[d]);
		}

		for (d = 0; d < pb.SS.D; d++)  
		{				
			SW.V[s].v[d] = 
				(alea( pb.SS.min[d], pb.SS.max[d] ) - SW.X[s].c[d])/2; 
		} 	
	}	

	// First evaluations
	for (s = 0; s < SW.S; s++) 
	{	
		SW.X[s].error = errorTot(h,g,kSup,SW.X[s]);
		SW.P[s] = SW.X[s];	// Best position = current one
	}

	nEval = SW.S;

	// Find the best
	SW.best = 0;
	errorPrev = fabs(pb.epsilon-SW.P[SW.best].error);

	for (s = 1; s < SW.S; s++)     
	{
		zz=fabs(pb.epsilon-SW.P[s].error);
		if (zz < errorPrev)
			SW.best = s;
		errorPrev=zz;			
	}
	initLinks = 1;		// So that information links will beinitialized
	// Note: It is also a flag saying "No improvement"
	noStop = 0;	

	// ---------------------------------------------- ITERATIONS
	iter=0; iterBegin=0;
	while (noStop == 0) 
	{
		iter=iter+1;
		// Who informs who, at random
		for (s = 0; s < SW.S; s++)
		{	
			for (m = 0; m < SW.S; m++)
			{		    
				if (alea (0, 1)<p) LINKS[m][s] = 1;	// Probabilistic method
				else LINKS[m][s] = 0;
			}

			// Each particle informs itself
			for (m = 0; m < SW.S; m++)
			{
				LINKS[m][m] = 1;	     
			}	  
		}

		// The swarm MOVES
		//printf("\nIteration %i",iter);
		for (s = 0; s < SW.S; s++)  index[s]=s;

		switch (param.randOrder)
		{
			default:
				break;

				case 1: //Define a random permutation
					length=SW.S;
				for (s=0;s<length;s++) indexTemp[s]=index[s];

				for (s=0;s<SW.S;s++)
			{
				rank=alea_integer(0,length-1);
				index[s]=indexTemp[rank];
				if (rank<length-1)	// Compact
				{
					for (t=rank;t<length;t++)
						indexTemp[t]=indexTemp[t+1];
				}					
				length=length-1;
			}
				break;			
		}

		for (s0 = 0; s0 < SW.S; s0++)	// For each particle ...
		{	
			s=index[s0];
			// ... find the first informant
			s1 = 0;    
			while (LINKS[s1][s] == 0)	s1++;					
			if (s1 >= SW.S)	s1 = s;

			// Find the best informant			
			gg = s1;	
			for (m = s1; m < SW.S; m++) 
			{	    
				if (LINKS[m][s] == 1 && SW.P[m].error < SW.P[gg].error)
					gg = m;
			}	

			//.. compute the new velocity, and move

			// Exploration tendency
			for (d = 0; d < pb.SS.D; d++)
			{
				SW.V[s].v[d]=param.w[0] *SW.V[s].v[d];
			}

			// Prepare Exploitation tendency
			for (d = 0; d < pb.SS.D; d++)
			{
				PX.v[d]= SW.P[s].c[d] - SW.X[s].c[d];
			}				
			PX.size=pb.SS.D; 

			if(gg!=s)
			{
				for (d = 0; d < pb.SS.D; d++)
				{
					GX.v[d]= SW.P[gg].c[d] - SW.X[s].c[d];
				}
				GX.size=pb.SS.D;
			}

			// Apply tendencies			
			for (d = 0; d < pb.SS.D; d++)
			{	
				SW.V[s].v[d]=SW.V[s].v[d] +
					+	alea(0, param.c1)*PX.v[d];
			}

			if (gg!=s)
			{
				for (d = 0; d < pb.SS.D; d++)
				{	
					SW.V[s].v[d]=SW.V[s].v[d] 
						+	alea(0,param.c1) * GX.v[d];			
				}
			}						

			// Update the position
			for (d = 0; d < pb.SS.D; d++)
			{	
				SW.X[s].c[d] = SW.X[s].c[d] + SW.V[s].v[d];			
			}

			if (nEval >= pb.evalMax) goto end;
			// --------------------------
			noEval = 1;

			// Set to the bounds, and v to zero
			for (d = 0; d < pb.SS.D; d++)
			{	
				if (SW.X[s].c[d] < pb.SS.min[d])
				{	
					SW.X[s].c[d] = pb.SS.min[d];
					SW.V[s].v[d] = 0;
				}

				if (SW.X[s].c[d] > pb.SS.max[d])
				{			
					SW.X[s].c[d] = pb.SS.max[d];
					SW.V[s].v[d] = 0;			
				}				
			}

			SW.X[s].error = errorTot(h,g,kSup,SW.X[s]);
			nEval = nEval + 1;				

			// ... update the best previous position
			if (SW.X[s].error < SW.P[s].error)	// Improvement
			{		
				SW.P[s] = SW.X[s];

				// ... update the best of the bests
				if (SW.P[s].error < SW.P[SW.best].error)
				{		
					SW.best = s;			
				}				
			}		
		}			// End of "for (s=0 ...  "	
		// Check if finished

		error = SW.P[SW.best].error;
		error= fabs(error - pb.epsilon);

		if (error < errorPrev)	// Improvement
		{		
			initLinks = 0;							
		}
		else			// No improvement
		{			
			initLinks = 1;	// Information links will be	reinitialized	
		}

		if(param.initLink==1) initLinks=1-initLinks;

		errorPrev = error;
		end:					
			switch (param.stop)
		{		
			case 0:			
				if (error > pb.epsilon && nEval < pb.evalMax)
					noStop = 0;	// Won't stop
				else
					noStop = 1;	// Will stop
				break;

			case 1:			
				if (nEval < pb.evalMax)
					noStop = 0;	// Won't stop
				else
					noStop = 1;	// Will stop
				break;		
		}

	} // End of "while nostop ...

	// struct model {int model;int size;double c[CMax];double error;};

	R.error = error;
	R.size=pb.SS.D;
	for(d=0;d<R.size;d++) R.c[d]=SW.P[SW.best].c[d];
	return R;  
}

// ===========================================================
double errorTot(double h[],double g[],int kSup,struct model c)
{
	double error;
	int k;
	double zz;

	error=0;
	for(k=0;k<=kSup;k++)
	{
		zz=modelG(h[k],c)-g[k];
		error=error+zz*zz;
	}
	return sqrt(error);
}

// ===========================================================
double modelG( double h, struct model c)
{
	// Estimate the fitness value g(h), according to the model

	double g=0; // Set to zero just for my stupid compiler
	double zz;

	switch(c.model)
	{
		case 0: // Power
			g=c.c[0]*pow(h,c.c[1]);
		break;

		case 1: // Exponential
			if(fabs(c.c[1])>zero) g=c.c[0]*(1-exp(-h/c.c[1]));
		else g=c.c[0];
		break;	

		case 2: // "Parabolic"
			if(h<=c.c[1])
		{
			if(fabs(c.c[1])>zero) g=c.c[0]*(1-pow(1-h/c.c[1],c.c[2]));	
			else g=c.c[0];
		}
		else
		{
			if(hMax-h>zero) g=c.c[0]*(1-pow((h-c.c[1])/(hMax-h),c.c[3]));	
			else g=0;	 
		}
		break;

		case 3: // Bell curve
			zz=
			pow(c.c[0],c.c[0])*pow(c.c[1],c.c[1])*pow(hMax/(c.c[0]+c.c[1]),c.c[0]+c.c[1]);	
		g=pow(h,c.c[0])*pow(hMax-h,c.c[1])*gMax/zz;
		break;

	}
	return g;
}
